# Written by Alex S. Hill (ashill@astro.ubc.ca)
# 2018-6-12
# This is a simplified example python code used to read and produce simple
#   visualizations of the data presented by
#   Hill, Mac Low, Gatto, & IbÃ¡Ã±ez-MejÃ­a, 2018, AAS Journals, submitted
# The code produces images similar to individual panels in Figures 1-4 and 7 of
#   Hill et al.
# The data and this code are available at doi: 10.5531/sd.astro.2
# This example reads our fiducial model, sn34 smd132 pe300 2 pc.
# The code requires that the yt (http://yt-project.org) and numpy packages be
#   installed. This code has been tested using python 3.6.4 and yt 3.4.1. The
#   code expects the data file to be in the current directory and outputs PNG
#   files with the images to the current directory.
#
# Copyright (c) 2018, Alex S. Hill
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met: 
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import yt
import numpy as np

# define fields for number density and P_thermal / k
def _n(field, data):
      return data["density"] / (1.375*yt.physical_constants.mp)
yt.add_field("n", function=_n, units='cm**-3', display_name = r'$n$')

def _pk(field, data):
      return data["n"] * data["temperature"]
yt.add_field("pk", function=_pk, units='K/cm**3', display_name=r'$P_\mathrm{th}/k$')

# gravitational acceleration
def g(z, a1 = (1.42e-3 * yt.units.kpc * (yt.units.Myr)**(-2.0)).in_cgs(),
      a2= (5.49e-4 * (yt.units.Myr)**-2.0).in_cgs(),
      a3 = (5e-5* (yt.units.kpc)**-1.0 * (yt.units.Myr)**-2.0).in_cgs(),
      z0 = (0.18*yt.units.kpc).in_cgs()):
    g = -a1*z / (np.sqrt(z**2 + z0**2)) - a2*z + a3*z*np.abs(z)
    return g

# load the data
ds = yt.load('sn34_smd132_pe300_2pc_hdf5_plt_cnt_0200')

# plot midplane slice of number density
f = yt.SlicePlot(ds, 'z', ['n', 'temperature', 'pk'])
f.set_width([(1,'kpc'),(1,'kpc')])
f.set_origin('center-domain')
f.set_zlim('n', 1e-5, 1e2)
f.set_zlim('temperature', 10, 1e8)
f.set_zlim('pk', 1e2,1e5)
f.save()

# plot vertical slice of number density, extending to z = +/- 2.5 kpc
f = yt.SlicePlot(ds, 'x', ['n', 'temperature', 'pk'])
f.set_width([(1,'kpc'),(5,'kpc')])
f.set_origin('center-domain')
f.set_zlim('n', 1e-5, 1e2)
f.set_zlim('temperature', 10, 1e8)
f.set_zlim('pk', 1e2,1e5)
f.save()

# plot column density looking face-on
f = yt.ProjectionPlot(ds, 'z', ['n'])
f.set_width([(1,'kpc'),(1,'kpc')])
f.set_origin('center-domain')
# set color bar label. These commands have to be last.
f._setup_plots()
f.plots['n'].cb.set_label(r'column density (cm$^{-2}$)')
f.save()

# calculate box extending to +/- 20 pc above the midplane
box = ds.box([-500, -500, -20]*yt.units.pc.in_cgs(), [500, 500, 20]*yt.units.pc.in_cgs())
f = yt.PhasePlot(box, 'n', 'pk', ['cell_volume'], weight_field=None,
                fractional=True, x_bins=128, y_bins=128)
f.save()

alldata = ds.all_data()
# midplane weight; average of weight felt from above and below the plane
area = (1*yt.units.kpc)**2
weight = np.sum(np.abs(g(alldata['z'])) * alldata['density'] * \
    alldata['dx']*alldata['dy']*alldata['dz']) / area / 2.0

weight = weight.in_units('dyne/cm**2')
print("weight = ", weight)
print("weight in P/k units = ",
      (weight / yt.physical_constants.boltzmann_constant_cgs).in_units('K/cm**3'))
